# Copyright (C) 2016  Carlos Henrique Silva <carlosqsilva@outlook.com>
#
# This library is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from .ccharts import ccharts
import numpy as np
from .tables import d2


class cusum(ccharts):

    _title = "CUSUM Chart"

    def __init__(self, target=None, std=None, interval=4):
        super(cusum, self).__init__()

        self.target = target
        self.std = std
        self.interval = interval

    def plot(self, data, size, newdata=None):

        if size > 1:
            data = np.mean(data, axis=1)

        target = self.target
        std = self.std
        interval = self.interval

        if target is None:
            target = np.mean(data)

        if std is None:
            rbar = []
            for i in range(len(data) - 1):
                rbar.append(abs(data[i] - data[i + 1]))
            std = np.mean(rbar) / d2[2]

        k = std / 2

        cplus = []  # values
        cminus = []  # values
        i, j = 0, 0
        for xi in data:
            cplus.append(max([0, xi - (target + k) + i]))
            cminus.append(min([0, xi - (target - k) + j]))
            i, j = cplus[-1], cminus[-1]

        lcl = -interval * std
        ucl = interval * std
        center = 0

#        ax.plot([0, len(cplus)], [center, center], 'k-')
#        ax.plot([0, len(cplus)], [lcl, lcl], 'r:')
#        ax.plot([0, len(cplus)], [ucl, ucl], 'r:')
#        ax.plot(cplus, 'bo--')
#        ax.plot(cminus, 'bo--')

        return ([cplus, cminus], center, lcl, ucl, self._title)



#!/usr/bin/env python3
#
# Copyright (C) 2016  Carlos Henrique Silva <carlosqsilva@outlook.com>
#
# This library is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import numpy as np
import matplotlib.ticker as mtick


class PlotCharts(object):
    """docstring for PlotCharts"""

    def __init__(self, ax, values, center, lcl, ucl, title):
        super(PlotCharts, self).__init__()

        self.plot_chart(ax, values, center, lcl, ucl, title)

    def plot_chart(self, ax, values, center, lcl, ucl, title, newvalues=None):

        ax.yaxis.tick_right()
        ax.yaxis.set_major_formatter(mtick.FormatStrFormatter('%.3f'))

        num = len(values)
        if isinstance(values[0], list):
            num = len(values[0])

        if newvalues:
            ax.plot([num - 0.5] * 2, [lcl, ucl], 'k--')
            ax.plot(values + newvalues, 'bo--')
            num += len(newvalues)

        newx = list(range(num))
        newx[0] = -0.3
        newx[-1] = num - 0.6

        if isinstance(lcl, list) and isinstance(ucl, list):
            ax.yaxis.set_ticks([center])
            ax.plot([-0.3, num], [center, center], 'k-')
            ax.plot(values, 'bo--')
            ax.fill_between(newx, lcl, ucl, facecolor='green', alpha=0.4, step='mid')
            ax.step(newx, lcl, 'r:', where='mid')
            ax.step(newx, ucl, 'r:', where='mid')

        else:
            ax.fill_between([-0.3, num], [lcl, lcl], [ucl, ucl], facecolor='green', alpha=0.4)
            ax.yaxis.set_ticks([lcl, center, ucl])
            ax.plot([0, num], [center, center], 'k-')
            ax.plot([0, num], [lcl, lcl], 'r:')
            ax.plot([0, num], [ucl, ucl], 'r:')

            if isinstance(values[0], list):
                ax.plot(values[0], 'bo--')
                ax.plot(values[1], 'bo--')
            else:
                ax.plot(values, 'bo--')

        # Set the title
        ax.set_title(title)

        # Change de y limits of the graph
        ylim = ax.get_ylim()
        factor = 0.2
        new_ylim = (ylim[0] + ylim[1]) / 2 + np.array((-0.5, 0.5)) * (ylim[1] - ylim[0]) * (1 + factor)
        if lcl == 0:
            ax.set_ylim([0, new_ylim[1]])
        else:
            ax.set_ylim(new_ylim)

        # Change x ticks
        new_xlim = [0, num]
        ax.set_xlim([0, num] + np.array((-0.3, -0.6)))
        ax.xaxis.set_ticks(np.arange(*new_xlim, 2))



#!/usr/bin/env python3
#
# Copyright (C) 2016  Carlos Henrique Silva <carlosqsilva@outlook.com>
#
# This library is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# from __future__ import (absolute_import, division, print_function,
#                         unicode_literals)
from .results import PlotCharts

import matplotlib.pyplot as plt
import matplotlib as mpl

import numpy as np
import pandas as pd

class spc(object):
    """
    spc is the main class of the library. It receive the data, plot the chart
    drop values and save the image to a file.

    :param data: Can be a list, nested list, numpy.array or pandas.Dataframe
    :param newdata: the same of above

    :Example:

    >>> import numpy
    >>> from pyspc import *
    >>> fake_data = numpy.random.randn(30, 5) + 100
    >>> chart1 = spc(fake_data) + xbar_rbar() + rbar() + rules()
    >>> print(chart1)

    """

    _title = 'SPC : Statistical Process Control Charts for Humans'

    def __init__(self, data=None, newdata=None):

        if isinstance(data, pd.DataFrame):
            data = data.values
        if isinstance(newdata, pd.DataFrame):
            newdata = newdata.values

        try:
            self.size = len(data[0])
        except:
            if not isinstance(data[0], (list, tuple, np.ndarray)):
                self.size = 1

        self.data = data
        self.newdata = newdata
        self.layers = []
        self.points = None
        self.summary = []

    def __repr__(self):
        self.make()
        plt.show()
        return "<pyspc: (%d)>" % self.__hash__()

    def __getitem__(self, i):
        return self.summary[i]

    def __iter__(self):
        for x in self.summary:
            yield x

    def get_subplots(self):
        if len(self.layers) > 1:
            return self.subplots[0]
        return self.subplots

    def save(self, filename, **kwargs):
        """
        Save the chart to a image file.

        :param filename: name of the image file, if no extenssion is provide it will be save as '.png'.
        :param **kwargs: see matplotlib.figure.Figure.savefig for more details.
        """
        if len(self.summary) == 0:
            self.make()

        self.fig.savefig(filename, **kwargs)

    def drop(self, *args):
        self.data = np.delete(self.data, args, axis=0)

    def make(self, **kwargs):
        num_layers = len(self.layers)
        if num_layers == 0:
            plt.show()
            return

        self.fig, *self.subplots = plt.subplots(num_layers, **kwargs)
        self.fig.canvas.set_window_title(self._title)

        for layer, ax in zip(self.layers, self.get_subplots()):
            summary = {}

            values, center, lcl, ucl, title = layer.plot(self.data, self.size, self.newdata)
            PlotCharts(ax, values, center, lcl, ucl, title)

            summary['name'] = title
            summary['values'] = values
            summary['lcl'] = lcl
            summary['ucl'] = ucl
            summary['center'] = center

            if self.points is not None:
                summary['violation-points'] = self.points.plot_violation_points(ax, values, center, lcl, ucl)

            self.summary = summary

            print(summary)

        self.fig.tight_layout()



# Copyright (C) 2016  Carlos Henrique Silva <carlosqsilva@outlook.com>
#
# This library is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from .ccharts import ccharts
from .tables import A3, B3, B4
import numpy as np


class xbar_sbar(ccharts):

    _title = "Xbar-S Chart"

    def plot(self, data, size, newdata=None):

        assert size >= 2
        assert size <= 10
        newvalues = None

        X, S = [], []
        for xs in data:
            assert len(xs) == size
            S.append(np.std(xs, ddof=1))
            X.append(np.mean(xs))

        if newdata:
            newvalues = [np.mean(xs) for xs in newdata]

        sbar = np.mean(S)
        xbar = np.mean(X)

        lclx = xbar - A3[size] * sbar
        uclx = xbar + A3[size] * sbar

        return (X, xbar, lclx, uclx, self._title)


class sbar(ccharts):

    _title = "Standard Deviation Chart"

    def plot(self, data, size, newdata=None):

        assert size >= 2
        assert size <= 10
        newvalues = None

        S = []
        for xs in data:
            assert len(xs) == size
            S.append(np.std(xs, ddof=1))

        if newdata:
            newvalues = [np.std(xs, ddof=1) for xs in newdata]

        sbar = np.mean(S)

        lcls = B3[size] * sbar
        ucls = B4[size] * sbar

        return (S, sbar, lcls, ucls, self._title)



# Copyright (C) 2016  Carlos Henrique Silva <carlosqsilva@outlook.com>
#
# This library is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from .ccharts import ccharts
from .tables import B3, B4
import numpy as np


class imrstd(ccharts):

    _title = "Standard Deviation Chart"

    def __init__(self, sizecol=1):
        super(imrstd, self).__init__()

        self.size = sizecol - 1

    def plot(self, data, size, newdata=None):

        sizes, data = data.T
        if self.size == 1:
            sizes, data = data, sizes

        samples = dict()
        for n, value in zip(sizes, data):
            if n in samples:
                samples[n].append(value)
            else:
                samples[n] = [value]

        sample_size = len(samples[1])
#        num_samples = len(samples)

        sample_std = []
        for key in samples:
            assert sample_size == len(samples[key])
            sample_std.append(np.std(samples[key], ddof=1))

        sbar = np.mean(sample_std)
        ucl_std = B4[sample_size] * sbar
        lcl_std = B3[sample_size] * sbar

        return (sample_std, sbar, lcl_std, ucl_std, self._title)



# Copyright (C) 2016  Carlos Henrique Silva <carlosqsilva@outlook.com>
#
# This library is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from .ccharts import ccharts
import numpy as np
from .tables import h4


class ewma(ccharts):

    _title = "EWMA Chart"

    def __init__(self, target=None, weight=0.2):
        super(ewma, self).__init__()

        self.target = target
        self.weight = weight

    def plot(self, data, size, newdata=None):
        assert ((self.weight > 0) and (self.weight < 1))

        if size > 1:
            data = np.mean(data, axis=1)

        target = self.target
        weight = self.weight

        # calculate the target with not given
        if target is None:
            target = np.mean(data)

        # calculate the standard deviation
        rbar = []
        for i in range(len(data) - 1):
            rbar.append(abs(data[i] - data[i + 1]))
        std = np.mean(rbar) / 1.128

        ewma = []  # values
        i = target
        for x in data:
            ewma.append(weight * x + (1 - weight) * i)
            i = ewma[-1]

        lcl, ucl = [], []
        for i in range(1, len(data) + 1):
            lcl.append(target - 3 * (std) * np.sqrt((weight / (2 - weight)) * (1 - (1 - weight)**(2 * i))))
            ucl.append(target + 3 * (std) * np.sqrt((weight / (2 - weight)) * (1 - (1 - weight)**(2 * i))))

#        ax.plot([0, len(ewma)], [target, target], 'k-')
#        ax.plot(lcl, 'r:')
#        ax.plot(ucl, 'r:')
#        ax.plot(ewma, 'bo--')

        return (ewma, target, lcl, ucl, self._title)

class mewma(ccharts):

    _title = "MEWMA Chart"

    def __init__(self, lambd=0.1):
        super(mewma, self).__init__()

        self.lambd = lambd

    def plot(self, data, size, newdata=None):

        nrow, ncol = data.shape
        mean = data.mean(axis=0)

        v = np.zeros(shape=(nrow - 1, ncol))
        for i in range(nrow - 1):
            v[i] = data[i + 1] - data[i]

        vv = v.T @ v

        s = np.zeros(shape=(ncol, ncol))
        for i in range(ncol):
            s[i] = (1 / (2 * (nrow - 1))) * (vv[i])

        mx = data - mean

        z = np.zeros(shape=(nrow + 1, ncol))
        for i in range(nrow):
            z[i + 1] = self.lambd * mx[i] + (1 - self.lambd) * z[i]
        z = z[1:, :]

        t2 = [] # values
        for i in range(nrow):
            w = (self.lambd / (2 - self.lambd)) * (1 - (1 - self.lambd)**(2 * (i + 1)))
            inv = np.linalg.inv(w * s)
            t2.append((z[i].T @ inv) @ z[i])

        ucl = h4[int(self.lambd * 10) - 1][ncol - 1]
        return(t2, 0, 0, ucl, self._title)



# Copyright (C) 2016  Carlos Henrique Silva <carlosqsilva@outlook.com>
#
# This library is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from .ccharts import ccharts
from .tables import d2
import numpy as np


class imrx(ccharts):

    _title = "X Chart"

    def __init__(self, sizecol=1):
        super(imrx, self).__init__()

        self.size = sizecol - 1

    def plot(self, data, size, newdata=None):

        sizes, data = [i for i in range(len(data))], data.T
        if self.size == 1:
            sizes, data = data, sizes

        samples = dict()
        for n, value in zip(sizes, data):
            if n in samples:
                samples[n].append(value)
            else:
                samples[n] = [value]

        sample_size = len(samples[1])
#        num_samples = len(samples)

        sample_mean = []  # VALUES
        sample_mr = []
        for key in samples:
            assert sample_size == len(samples[key])
            sample_mean.append(np.mean(samples[key]))
            try:
                v1, v2 = sample_mean[-2:]
                sample_mr.append(abs(v1 - v2))
            except:
                sample_mr.append(np.nan)

        xbar = np.mean(sample_mean)
        mrbar = np.nanmean(sample_mr)

        ucl_xbar = xbar + 3 * (mrbar / d2[2])
        lcl_xbar = xbar - 3 * (mrbar / d2[2])

        return (sample_mean, xbar, lcl_xbar, ucl_xbar, self._title)



#!/usr/bin/env python3
from pyspc import *

a = spc(pistonrings) + cusum() + ewma() + rules()
#print(a)
b = spc(pistonrings) + xbar_sbar() + sbar() + rules()
#print(b)
c = spc(viscosidade) + xmr() + mr() + cusum() + rules()
#print(c)
d = spc(plastic) + Tsquare_single() + rules()
#print(d)
e = spc(experiment) + Tsquare()
#print(e)
f = spc(mewma_example) + mewma() + rules()
#print(f)



# Copyright (C) 2016  Carlos Henrique Silva <carlosqsilva@outlook.com>
#
# This library is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from .ccharts import ccharts
from .tables import D4, D3, d2
import numpy as np


class mr(ccharts):

    _title = "MR - Moving Range Chart"

    def plot(self, data, size, newdata=None):
        assert size == 1
        newvalues = None

        R = np.array([np.nan] + [abs(data[i] - data[i + 1]) for i in range(len(data) - 1)])

        if newdata:
            newdata = data[-1:] + newdata
            n = len(newdata)
            newvalues = [abs(newdata[i] - newdata[i + 1]) for i in range(n - 1)]

        Rbar = np.nanmean(R)

        lclr = D3[2] * Rbar
        uclr = D4[2] * Rbar

        return (R, Rbar, lclr, uclr, self._title)


class xmr(ccharts):

    _title = "X chart"

    def plot(self, data, size, newdata=None):
        assert size == 1
        newvalues = None

        R = np.array([np.nan] + [abs(data[i] - data[i + 1]) for i in range(len(data) - 1)])

        if newdata:
            newvalues = newdata

        Rbar = np.nanmean(R)
        Xbar = np.mean(data)

        lclx = Xbar - 3 * (Rbar / d2[2])
        uclx = Xbar + 3 * (Rbar / d2[2])

        return (data, Xbar, lclx, uclx, self._title)



#!/usr/bin/env python3
#
# Copyright (C) 2016  Carlos Henrique Silva <carlosqsilva@outlook.com>
#
# This library is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from .pyspc import spc


class rules(object):

    def __init__(self, rules='BASIC'):
        self.layers = self
        self.rules = rules

        # default : BASIC
        if rules.lower() == 'basic':
            self.rules = [self.RULE_7_ON_ONE_SIDE,
                          self.RULE_1_BEYOND_3SIGMA]
        # WECO
#        if rules == 2:
#            self.rules = [self.RULES_1_BEYOND_3SIGMA,
#                          self.RULES_2_OF_3_BEYOND_2SIGMA,
#                          self.RULES_4_OF_5_BEYOND_1SIGMA,
#                          self.RULES_8_ON_ONE_SIDE,
#                          self.RULES_6_TRENDING, RULES_14_UP_DOWN]

    def __radd__(self, model):
        if isinstance(model, spc):
            model.points = self.layers
            return model

        self.layers.append(model)
        return self

    def test_violating_runs(self, data, center, lcl, ucl):
        for i in range(1, len(data)):
            if (data[i - 1] - center) * (data[i] - center) < 0:
                return False
        return True

    def test_beyond_limits(self, value, lcl, ucl):
        return value > ucl or value < lcl

    def RULE_1_BEYOND_3SIGMA(self, ax, values, center, lcl, ucl):
        points = []
        if isinstance(lcl, list) and isinstance(ucl, list):
            for i, value in enumerate(values):
                if self.test_beyond_limits(value, lcl[i], ucl[i]):
                    ax.plot([i], value, 'ro', markersize=5)
                    points.append(i)

        elif isinstance(values[0], list):
            for i in range(len(values)):
                for j, value in enumerate(values[i]):
                    if self.test_beyond_limits(value, lcl, ucl):
                        ax.plot([j], value, 'ro', markersize=5)
                        points.append(j)
        else:
            for i in range(len(values)):
                if self.test_beyond_limits(values[i], lcl, ucl):
                    ax.plot([i], values[i], 'ro', markersize=5)
                    points.append(i)

        return points

    def RULE_7_ON_ONE_SIDE(self, ax, values, center, lcl, ucl):
        points = []
        # ewma, p Charts
        if isinstance(lcl, list) or isinstance(values[0], list):
            return points
        # mewma Chart
        if center == 0:
            return points

        num = 7
        for i in range(len(values)):
            if i <= (num - 1):
                continue
            if self.test_violating_runs(values[i - num + 1:i + 1], center, lcl, ucl):
                ax.plot([i], values[i], 'yo', markersize=5)
                points.append(i)

        return points

    def plot_violation_points(self, ax, values, center, lcl, ucl):
        violating_points = []
        for func in self.rules:
            violating_points += func(ax, values, center, lcl, ucl)
        return list(set(violating_points))



#!/usr/bin/env python3
#
# Copyright (C) 2016  Carlos Henrique Silva <carlosqsilva@outlook.com>
#
# This library is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import (absolute_import, division, print_function,
                        unicode_literals)
__version__ = '1.0.8'
__author__ = "Brendan Perry"
__email__ = "bperry@hey.com"

import warnings
warnings.simplefilter("ignore")

# For testing purposes we might need to set mpl backend before any
# other import of matplotlib.
# def _set_mpl_backend():
#     import os
#     import matplotlib as mpl

#     env_backend = os.environ.get('MATPLOTLIB_BACKEND')
#     if env_backend:
#         # we were instructed
#         mpl.use(env_backend)

#_set_mpl_backend()


from .ccharts import *
from .pyspc import spc
from .rules import rules



# Copyright (C) 2016  Carlos Henrique Silva <carlosqsilva@outlook.com>
#
# This library is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from .ccharts import ccharts
from .tables import D3, D4
import numpy as np


class imrr(ccharts):

    _title = "R Chart"

    def __init__(self, sizecol=1):
        super(imrr, self).__init__()

        self.size = sizecol - 1

    def plot(self, data, size, newdata=None):

        sizes, data = data.T
        if self.size == 1:
            sizes, data = data, sizes

        samples = dict()
        for n, value in zip(sizes, data):
            if n in samples:
                samples[n].append(value)
            else:
                samples[n] = [value]

        sample_size = len(samples[1])
#        num_samples = len(samples)

        sample_r = []
        for key in samples:
            assert sample_size == len(samples[key])
            sample_r.append(max(samples[key]) - min(samples[key]))

        rbar = np.mean(sample_r)
        ucl_rbar = D4[sample_size] * rbar
        lcl_rbar = D3[sample_size] * rbar

#        ax.plot([0, num_samples], [rbar, rbar], 'k-')
#        ax.plot([0, num_samples], [lcl_rbar, lcl_rbar], 'r:')
#        ax.plot([0, num_samples], [ucl_rbar, ucl_rbar], 'r:')
#        ax.plot(sample_r, 'bo--')

        return (sample_r, rbar, lcl_rbar, ucl_rbar, self._title)



# -*- coding: utf-8 -*-
import pandas as pd
import os

_ROOT = os.path.abspath(os.path.dirname(__file__))

pistonrings = pd.read_csv(os.path.join(_ROOT, 'pistonrings.csv')).values

diameter = pd.read_csv(os.path.join(_ROOT, 'diameter.csv')).values

parts = pd.read_csv(os.path.join(_ROOT, 'parts.csv')).values

sizes = pd.read_csv(os.path.join(_ROOT, 'sizes.csv')).values

chemical = pd.read_csv(os.path.join(_ROOT, 'chemical.csv')).values

newproduct = pd.read_csv(os.path.join(_ROOT, 'newproduct.csv')).values

waitingTime = pd.read_csv(os.path.join(_ROOT, 'waitingTime.csv')).values

sandbox = pd.read_csv(os.path.join(_ROOT, 'sandbox.csv')).values

canjuice = pd.read_csv(os.path.join(_ROOT, 'canjuice.csv')).values

circuits = pd.read_csv(os.path.join(_ROOT, 'circuits.csv')).values

inspection = pd.read_csv(os.path.join(_ROOT, 'inspection.csv')).values

paint = pd.read_csv(os.path.join(_ROOT, 'PaintMaterial.csv')).values

thickness = pd.read_csv(os.path.join(_ROOT, 'thickness.csv')).values

viscosity = pd.read_csv(os.path.join(_ROOT, 'Viscosidade.csv')).values

plastic = pd.read_csv(os.path.join(_ROOT, 'hotelling.csv')).values

experiment = pd.read_csv(os.path.join(_ROOT, 'experiment.csv')).values

mewma_example = pd.read_csv(os.path.join(_ROOT, 'dados.csv')).values



# Copyright (C) 2016  Carlos Henrique Silva <carlosqsilva@outlook.com>
#
# This library is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from .ccharts import ccharts
import numpy as np


class u(ccharts):

    _title = "U Chart"

    def __init__(self, size=1):
        super(u, self).__init__()

        self.size = size - 1

    def plot(self, data, size, newdata=None):

        sizes, data = data.T
        if self.size == 1:
            sizes, data = data, sizes
            print('oi')

        data2 = sizes / data
        ubar = np.sum(sizes) / np.sum(data)

        lcl, ucl = [], []
        for i in data:
            lcl.append(ubar - 3 * np.sqrt(ubar / i))
            ucl.append(ubar + 3 * np.sqrt(ubar / i))

#        ax.plot([0, len(data2)], [ubar, ubar], 'k-')
#        ax.step(lcl, 'r', where='mid')
#        ax.step(ucl, 'r', where='mid')
#        ax.plot(data2, 'bo-')

        return (data2, ubar, lcl, ucl, self._title)



# Copyright (C) 2016  Carlos Henrique Silva <carlosqsilva@outlook.com>
#
# This library is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from .ccharts import ccharts
import numpy as np


class c(ccharts):

    _title = "C chart"

    def __init__(self, size=1):
        super(c, self).__init__()

        self.size = size - 1

    def plot(self, data, size, newdata=None):

        sizes, data = data.T
        if self.size == 1:
            sizes, data = data, sizes

        # the samples must have the same size for this charts
        assert np.mean(sizes) == sizes[0]

        cbar = np.mean(data)

        lcl = cbar - 3 * np.sqrt(cbar)
        ucl = cbar + 3 * np.sqrt(cbar)
#
#        ax.plot([0, len(data)], [cbar, cbar], 'k-')
#        ax.plot([0, len(data)], [lcl, lcl], 'r:')
#        ax.plot([0, len(data)], [ucl, ucl], 'r:')
#        ax.plot(data, 'bo-')

        return (data, cbar, lcl, ucl, self._title)



# Copyright (C) 2016  Carlos Henrique Silva <carlosqsilva@outlook.com>
#
# This library is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from .ccharts import ccharts
from .tables import D3, D4
import numpy as np


class imrmr(ccharts):

    _title = "Moving Range Chart"

    def __init__(self, sizecol=1):
        super(imrmr, self).__init__()

        self.size = sizecol - 1

    def plot(self, data, size, newdata=None):

        sizes, data = data.T
        if self.size == 1:
            sizes, data = data, sizes

        samples = dict()
        for n, value in zip(sizes, data):
            if n in samples:
                samples[n].append(value)
            else:
                samples[n] = [value]

        sample_size = len(samples[1])
#        num_samples = len(samples)

        sample_mr = []
        sample_mean = []
        for key in samples:
            assert sample_size == len(samples[key])
            sample_mean.append(np.mean(samples[key]))
            try:
                v1, v2 = sample_mean[-2:]
                sample_mr.append(abs(v1 - v2))
            except:
                sample_mr.append(np.nan)

        mrbar = np.nanmean(sample_mr)  # CENTER
        ucl_mr = D4[2] * mrbar
        lcl_mr = D3[2] * mrbar

#        ax.plot([0, num_samples], [mrbar, mrbar], 'k-')
#        ax.plot([0, num_samples], [lcl_mr, lcl_mr], 'r:')
#        ax.plot([0, num_samples], [ucl_mr, ucl_mr], 'r:')
#        ax.plot(sample_mr, 'bo--')

        return (sample_mr, mrbar, lcl_mr, ucl_mr, self._title)



#!/usr/bin/env python3
#
# Copyright (C) 2016  Carlos Henrique Silva <carlosqsilva@outlook.com>
#
# This library is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from ..pyspc import spc


class ccharts(object):

    def __init__(self):
        self.layers = [self]

    def __radd__(self, model):
        if isinstance(model, spc):
            model.layers += self.layers
            return model

        self.layers.append(model)
        return self



# Copyright (C) 2016  Carlos Henrique Silva <carlosqsilva@outlook.com>
#
# This library is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from .ccharts import ccharts
from.tables import B3, B4
from scipy.stats import beta, f
import numpy as np


def cova(mat):

    l, _ = mat.shape
    for i in range(1, l):
        cov = np.hstack((np.array([]), mat.diagonal(i)))
    return cov


def var_cov(var, s):

    varmean = var.mean(axis=0)
    smean = s.mean(axis=0)
    n = len(varmean)
    mat = np.zeros(shape=(n, n)) + np.diag(varmean)

    a, b = 0, n - 1
    for i in range(1, n):
        mat = mat + np.diag(smean[a:b], k=i) + np.diag(smean[a:b], k=-i)
        a, b = b, (b + (b - 1))

    return mat


class Tsquare_single(ccharts):

    _title = "T-square Hotelling Chart"

    def plot(self, data, size, newdata=None):

        data = np.array(data)
        numsample = len(data)

        colmean = np.mean(data, axis=0)
        matcov = np.cov(data.T)
        matinv = np.linalg.inv(matcov)

        values = []
        for sample in data:
            dif = sample - colmean
            value = matinv.dot(dif.T).dot(dif)
            values.append(value)

        cl = ((numsample - 1)**2) / numsample
        lcl = cl * beta.ppf(0.00135, size / 2, (numsample - size - 1) / 2)
        center = cl * beta.ppf(0.5, size / 2, (numsample - size - 1) / 2)
        ucl = cl * beta.ppf(0.99865, size / 2, (numsample - size - 1) / 2)

        return (values, center, lcl, ucl, self._title)


class Tsquare(ccharts):

    _title = "T-square Hotelling Chart"

    def plot(self, data, size, newdata=None):

        sizes = data[:, 0]
        sample = data[:, 1:]

        samples = dict()
        for n, value in zip(sizes, sample):
            if n in samples:
                samples[n] = np.vstack([samples[n], value])
            else:
                samples[n] = value

        m = len(samples.keys())
        n = len(samples[1])
        p = len(samples[1].T)

        variance, S = [], []
        for i in range(m):
            mat = np.cov(samples[i + 1].T, ddof=1)
            variance.append(mat.diagonal())
            S.append(cova(mat))

        variance, S = np.array(variance), np.array(S)

        means = np.array([samples[xs + 1].mean(axis=0) for xs in range(m)])
        means_total = means.mean(axis=0)

        Smat = var_cov(variance, S)
        Smat_inv = np.linalg.inv(Smat)

        values = []
        for i in range(m):
            a = means[i] - means_total
            values.append(5 * a @ Smat_inv @ a.T)

        p1 = (p * (m - 1) * (n - 1))
        p2 = (m * n - m - p + 1)
        lcl = (p1 / p2) * f.ppf(0.00135, p, p2)
        center = (p1 / p2) * f.ppf(0.50, p, p2)
        ucl = (p1 / p2) * f.ppf(0.99865, p, p2)

        return (values, center, lcl, ucl, self._title)


class variation(ccharts):

    _title = "Generalized Variance"

    def plot(self, data, size, newdata=None):

        mean = np.mean(data, axis=0)
        std = np.std(data, axis=0, ddof=1)
        svalues = []
        for sample in data:
            value = []
            for i in range(size):
                value.append((sample[i] - mean[i]) / std[i])
            a = sum([x * x for x in value])
            b = np.mean(value)
            s = np.sqrt((a - 3 * (b * b)) / 2)
            svalues.append(s)

        sbar = np.mean(svalues)
        lcl = B3[size + 1] * sbar
        ucl = B4[size + 1] * sbar

        return (svalues, sbar, lcl, ucl, self._title)



# Copyright (C) 2016  Carlos Henrique Silva <carlosqsilva@outlook.com>
#
# This library is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from .ccharts import ccharts
import numpy as np


class p(ccharts):

    _title = "P Chart"

    def __init__(self, size=1):
        super(p, self).__init__()

        self.size = size - 1

    def plot(self, data, size, newdata=None):

        sizes, data = data.T
        if self.size == 1:
            sizes, data = data, sizes

        data2 = data / sizes
        pbar = np.mean(data2)

        for n in sizes:
            assert n * pbar >= 5
            assert n * (1 - pbar) >= 5

        if np.mean(sizes) == sizes[0]:
            size = sizes[0]
            lcl = pbar - 3 * np.sqrt((pbar * (1 - pbar)) / size)
            ucl = pbar + 3 * np.sqrt((pbar * (1 - pbar)) / size)

            if lcl < 0:
                lcl = 0
            if ucl > 1:
                ucl = 1

            return (data2, pbar, lcl, ucl, self._title)

        else:
            lcl, ucl = [], []
            for size in sizes:
                lcl.append(pbar - 3 * np.sqrt((pbar * (1 - pbar)) / size))
                ucl.append(pbar + 3 * np.sqrt((pbar * (1 - pbar)) / size))

            return (data2, pbar, lcl, ucl, self._title)



# -*- coding: utf-8 -*-

from .cusum import cusum
from .ewma import ewma, mewma
from .mr import mr, xmr
from .xbar_sbar import xbar_sbar, sbar
from .xbar_rbar import xbar_rbar, rbar
from .imrx import imrx
from .imrr import imrr
from .imrmr import imrmr
from .imrstd import imrstd
from .hotelling import Tsquare_single, Tsquare, variation
from .np import np
from .p import p
from .c import c
from .u import u
from .tables import *



# Copyright (C) 2016  Carlos Henrique Silva <carlosqsilva@outlook.com>
#
# This library is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from .ccharts import ccharts
from .tables import A2, D3, D4
import numpy as np


class xbar_rbar(ccharts):

    _title = "Xbar-R Chart"

    def plot(self, data, size, newdata=None):
        assert size >= 2
        assert size <= 10
        newvalues = None

        R, X = [], []  # values
        for xs in data:
            assert len(xs) == size
            R.append(max(xs) - min(xs))
            X.append(np.mean(xs))

        if newdata:
            newvalues = [np.mean(xs) for xs in newdata]

        Rbar = np.mean(R)  # center
        Xbar = np.mean(X)

        lcl = Xbar - A2[size] * Rbar
        ucl = Xbar + A2[size] * Rbar

        return (X, Xbar, lcl, ucl, self._title)


class rbar(ccharts):

    _title = "R Chart"

    def plot(self, data, size, newdata=None):
        assert size >= 2
        assert size <= 10
        newvalues = None

        R = []  # values
        for xs in data:
            assert len(xs) == size
            R.append(max(xs) - min(xs))

        if newdata:
            newvalues = [max(xs) - min(xs) for xs in newdata]

        Rbar = np.mean(R)  # center

        lcl = D3[size] * Rbar
        ucl = D4[size] * Rbar

        return (R, Rbar, lcl, ucl, self._title)



#!/usr/bin/env python3
#
# Copyright (C) 2016  Carlos Henrique Silva <carlosqsilva@outlook.com>
#
# This library is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from setuptools import setup, find_packages

setup(name='pyspcalt',
      version='1.0.8',
      description='An altered version of pyspc from https://github.com/carlosqsilva/pyspc that does not break other graphs',
      long_description='An altered version of pyspc from https://github.com/carlosqsilva/pyspc that does not break other graphs',
      url='https://github.com/brendanperry/pyspc',
      author='Brendan Perry',
      author_email='bperry@hey.com',
      license='GPLv3',
      packages=find_packages(),
      package_dir={ "pyspcalt": "pyspcalt" },
      package_data={
      "pyspcalt": ["sampledata/*.csv"]},
      install_requires=['pandas', 'matplotlib', 'numpy', 'scipy'],
      test_suite="tests",
      keywords='SPC QCC CEQ CEP UEPA',
      classifiers=[
      'Development Status :: 4 - Beta',
      'Intended Audience :: Developers',
      'Intended Audience :: Science/Research',
      'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
      'Topic :: Scientific/Engineering',
      'Operating System :: Microsoft :: Windows',
      'Operating System :: POSIX :: Linux',
      'Operating System :: MacOS',
      'Programming Language :: Python :: 3.3',
      'Programming Language :: Python :: 3.4',
      'Programming Language :: Python :: 3.5',
      'Topic :: Software Development :: Libraries :: Python Modules'],
      zip_safe=False)
